from datetime import timezone

from obspy import UTCDateTime
from pydantic import BaseModel
from typing import List, Optional, Dict, Any

from .MetadataCategory import MetadataCategory
from ..pydantic_utcdatetime import CustomUTCDateTimeType


class MetadataQuery(BaseModel):
    id: int = None
    category: Optional[MetadataCategory] = None
    starttime: Optional[CustomUTCDateTimeType] = None
    endtime: Optional[CustomUTCDateTimeType] = None
    created_after: Optional[CustomUTCDateTimeType] = None
    created_before: Optional[CustomUTCDateTimeType] = None
    network: Optional[str] = None
    station: Optional[str] = None
    channel: Optional[str] = None
    location: Optional[str] = None
    data_valid: Optional[bool] = None
    metadata: Optional[Dict[str, Any]] = None
    status: Optional[List[str]] = None

    def datetime_dict(self, **kwargs):
        values = self.model_dump(**kwargs)
        for key in ["starttime", "endtime", "created_after", "created_before"]:
            if key in values and values[key] is not None:
                values[key] = values[key].datetime.replace(tzinfo=timezone.utc)
        return values
